//+------------------------------------------------------------------+
//|                                   Stochastic alb + speed MTF.mq4 |
// http://www.forex-tsd.com/advanced-elite/22848-adaptive-lookback-indicators.html#post313906
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers  2

#property indicator_minimum  0
#property indicator_maximum  100
#property indicator_color1   Lime
#property indicator_color2   Orange
#property indicator_style2   STYLE_DOT

extern string TimeFrame   = "current time frame";
extern int    swingCount = 5;
extern int    slowing    = 3;
extern int    signal     = 3;
extern int    signalMode = MODE_EMA;
extern int    price      = 0;
extern double speed      = 1.0;
extern double levelOs    = 20; // 38.2
extern double levelOb    = 80; // 61.8
extern bool   Interpolate = true;

double swingBuffer[];
double stoBuffer[];
double sigBuffer[];

int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculateValue;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int init(){
   IndicatorBuffers(3);
   SetIndexBuffer(0,stoBuffer);
   SetIndexBuffer(1,sigBuffer);
   SetIndexBuffer(2,swingBuffer);
   
   SetLevelValue(0,levelOs);
   SetLevelValue(1,levelOb);
   SetLevelValue(2,50);
   
   speed = MathMax(speed,0.000001);
   
   IndicatorFileName = WindowExpertName();
   CalculateValue    = (TimeFrame=="calculateSto"); if (CalculateValue) return(0);
   ReturningBars     = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
   IndicatorShortName("Stochastic alb+speed+MTF "+timeFrameToString(timeFrame));
   return(0);
}

int deinit(){
   return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start(){
   int counted_bars=IndicatorCounted();
   int i,limit,k,n;
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
   limit = Bars-counted_bars;
   
   if (ReturningBars)  { stoBuffer[0] = limit+1; return(0); }
   if (ReturningBars)  { sigBuffer[0] = limit+1; return(0); }
   
   if (CalculateValue || timeFrame == Period()) {         
      for(i=limit; i>=0; i--){
         int swing = 0;
         if (High[i]>High[i+1] && High[i+1]>High[i+2] && Low[i+2] < Low[i+3] && Low[i+3] < Low[i+4]) swing = -1;
         if (Low[i] < Low[i+1] && Low[i+1] < Low[i+2] && High[i+2]>High[i+3] && High[i+3]>High[i+4]) swing =  1;
         swingBuffer[i] = swing;
         k = i;
         n = 0;
         for (; (k<Bars) && (n<swingCount); k++) if(swingBuffer[k]!=0) n++;
         double albPeriod = MathMax(MathRound((k-i)/swingCount/speed),1);
         stoBuffer[i] = iStochastic(NULL,0,albPeriod,signal,slowing,signalMode,price,MODE_MAIN,i);
         sigBuffer[i] = iStochastic(NULL,0,albPeriod,signal,slowing,signalMode,price,MODE_SIGNAL,i);
      }
      return(0);
   }

// MTF
    
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i >= 0; i--){
      int y = iBarShift(NULL,timeFrame,Time[i]);
      stoBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateSto",swingCount,slowing,signal,signalMode,price,speed,levelOs,levelOb,MODE_MAIN,y); //speed,levelOs,levelOb,
      sigBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateSto",swingCount,slowing,signal,signalMode,price,speed,levelOs,levelOb,MODE_SIGNAL,y);
      
      if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
      if (!Interpolate) continue;
      datetime time = iTime(NULL,timeFrame,y);
      for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
      double factor = 1.0 / n;
      for(k = 1; k < n; k++){
         stoBuffer[i+k] = k*factor*stoBuffer[i+n] + (1.0-k*factor)*stoBuffer[i];
         sigBuffer[i+k] = k*factor*sigBuffer[i+n] + (1.0-k*factor)*sigBuffer[i];      
      }
   }  
   return(0);        
}

//+-------------------------------------------------------------------
//|  MTF subfunctions
//+-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs){
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
   return(Period());
}

string timeFrameToString(int tf){
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
   return("");
}

string StringUpperCase(string str){
   string   s = str;
   for (int length=StringLen(str)-1; length>=0; length--){
      int char = StringGetChar(s, length);
      if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
      else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}